"""Subclasses of this class decorate (i.e. "wrap") urlscheme modules."""

__docformat__ = "restructuredtext"

# Created: Sat Apr 30 18:32:38 PDT 2005
# Author: Shannon -jj Behrens
# Email: jjinux@users.sourceforge.net
#
# Copyright (c) Shannon -jj Behrens.  All rights reserved.

from aquarium.util.AquariumClass import AquariumClass


class UrlSchemeDecorator(AquariumClass):

    """Subclasses of this class decorate (i.e. "wrap") urlscheme modules.

    Whereas urlscheme modules are often very concerned with dealing with
    arbitrary Web server URL madness, urlscheme decorator modules are more
    concerned with aesthetic URLs.  Hence, you can have a urlscheme module, such
    as ``PathInfo``, which is appropriate for CGI under Apache, and you can
    wrap it with a mythical ``urlscheme.decorator.Backwards`` module that reads
    and writes the ``PathInfo`` backwards.  ``urlscheme.decorator.Backwards``
    could similarly wrap any of the other standard urlscheme modules.  In the
    example above, you would put the following in AquariumProperties::

        URL_SCHEME = "decorator.Backwards/PathInfo"

    Which is read as "decorator.Backwards over PathInfo".  Yes, you can
    decorate decorators ;)

    The following properties are used:

    inner
      This is the urlscheme instance this instance wraps.

    """

    def __init__(self, ctx, inner):
        """Accept ``ctx`` and ``inner``."""
        AquariumClass.__init__(self, ctx)
        self.inner = inner

    def __getattr__(self, attr):
        """Let ``self.inner`` take care of it."""
        return getattr(self.inner, attr)
